package com.chelvc.framework.base.config;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.function.Predicate;

import com.google.common.collect.Maps;
import lombok.NonNull;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.StringUtils;
import org.springframework.boot.autoconfigure.web.servlet.WebMvcRegistrations;
import org.springframework.core.io.Resource;
import org.springframework.web.servlet.mvc.method.annotation.RequestMappingHandlerMapping;

/**
 * 多服务MVC配置
 * <p>
 * 自动将Controller接口地址加上前缀（服务名称）
 *
 * @author Woody
 * @date 2023/1/1
 */
public class MultiserverMvcConfigurer implements WebMvcRegistrations {
    /**
     * 根据资源配置获取请求前缀，如果为空则不使用前缀
     *
     * @param resource 资源对象
     * @return 请求前缀
     */
    protected String getRequestPrefix(@NonNull Resource resource) {
        return ApplicationContextHolder.getApplicationName(resource).replace('.', '/');
    }

    @Override
    public RequestMappingHandlerMapping getRequestMappingHandlerMapping() {
        List<Resource> resources = ApplicationContextHolder.getApplicationResources();
        Map<String, Predicate<Class<?>>> prefixes = Maps.newHashMapWithExpectedSize(resources.size());
        for (Resource resource : resources) {
            String prefix = this.getRequestPrefix(resource);
            if (StringUtils.isEmpty(prefix)) {
                continue;
            }
            prefixes.put(prefix, clazz -> {
                String file = clazz.getProtectionDomain().getCodeSource().getLocation().getPath();
                try {
                    return resource.getURL().getPath().contains(file);
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
            });
        }
        RequestMappingHandlerMapping mapping = new RequestMappingHandlerMapping();
        mapping.setPathPrefixes(prefixes);
        return mapping;
    }
}
