package com.chelvc.framework.base.interceptor;

import java.util.Arrays;
import javax.servlet.http.HttpServletRequest;

import lombok.extern.slf4j.Slf4j;
import com.chelvc.framework.base.context.SessionContextHolder;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.boot.web.servlet.error.ErrorController;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.ExceptionHandler;

/**
 * 请求日志拦截器
 *
 * @author Woody
 * @date 2023/1/1
 */
@Slf4j
@Aspect
@Component
@Order(Ordered.HIGHEST_PRECEDENCE)
public class RequestLoggingInterceptor {
    /**
     * 判断是否可记录日志
     *
     * @param point 接口方法拦截点
     * @return true/false
     */
    private boolean isPrintable(JoinPoint point) {
        Object target = point.getTarget();
        if (target instanceof ErrorController) {
            return false;
        }
        Signature signature = point.getSignature();
        return signature instanceof MethodSignature
                && !((MethodSignature) signature).getMethod().isAnnotationPresent(ExceptionHandler.class);
    }

    /**
     * 接口请求日志打印
     *
     * @param point 接口方法拦截点
     * @throws Throwable 接口调用异常
     */
    @Around("@within(org.springframework.stereotype.Controller) " +
            "|| @within(org.springframework.web.bind.annotation.RestController)")
    public Object logging(ProceedingJoinPoint point) throws Throwable {
        HttpServletRequest request = SessionContextHolder.getRequest();
        boolean printable = this.isPrintable(point) && log.isDebugEnabled();
        if (printable) {
            log.debug(SessionContextHolder.getLoggingMessage(request, "REQUEST", Arrays.toString(point.getArgs())));
        }
        Object value = point.proceed();
        if (printable) {
            log.debug(SessionContextHolder.getLoggingMessage(request, "RESPONSE", String.valueOf(value)));
        }
        return value;
    }
}
