package com.chelvc.framework.base.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.Enumeration;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.google.common.collect.Maps;
import lombok.NonNull;
import com.chelvc.framework.common.util.HostUtils;
import com.chelvc.framework.common.util.StringUtils;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;

/**
 * Http工具类
 *
 * @author Woody
 * @date 2023/1/1
 */
public final class HttpUtils {
    /**
     * 资源分隔符匹配模式
     */
    private static final Pattern URI_SEPARATOR_PATTERN = Pattern.compile("/+");

    /**
     * 客户端主机地址请求头数组
     */
    private static final String[] REQUEST_ADDRESS_HEADER_ARRAY = new String[]{
            "X-Real-IP", "X-Forwarded-For", "Proxy-Client-IP", "WL-Proxy-Client-IP"
    };

    private HttpUtils() {
    }

    /**
     * 获取Http状态
     *
     * @param code Http状态码
     * @return Http状态枚举
     */
    public static HttpStatus getStatus(Integer code) {
        if (code == null) {
            return HttpStatus.INTERNAL_SERVER_ERROR;
        }
        try {
            return HttpStatus.valueOf(code);
        } catch (Exception e) {
            return HttpStatus.INTERNAL_SERVER_ERROR;
        }
    }

    /**
     * 获取请求资源
     *
     * @param request Http请求对象
     * @return 资源地址
     */
    public static String getRequestURI(@NonNull HttpServletRequest request) {
        String uri = request.getRequestURI();
        if (StringUtils.isEmpty(uri)) {
            return StringUtils.URI_SEPARATOR;
        } else if (uri.charAt(0) != StringUtils.URI_SEPARATOR.charAt(0)) {
            return StringUtils.URI_SEPARATOR.concat(uri);
        }
        return uri;
    }

    /**
     * 获取请求域名
     *
     * @param request Http请求对象
     * @return 域名
     */
    public static String getRequestDomain(@NonNull HttpServletRequest request) {
        return request.getScheme() + "://" + request.getServerName() +
                ":" + request.getServerPort() + request.getContextPath();
    }

    /**
     * 获取请求客户端主机地址
     *
     * @param request Http请求对象
     * @return 请求客户端主机地址
     */
    public static String getRequestAddress(@NonNull HttpServletRequest request) {
        // 获取请求主机地址
        String ip = null;
        for (String header : REQUEST_ADDRESS_HEADER_ARRAY) {
            if (StringUtils.nonEmpty(ip = request.getHeader(header)) && !HostUtils.UNKNOWN_ADDRESS.equals(ip)) {
                break;
            }
        }
        if (StringUtils.isEmpty(ip)) {
            ip = request.getRemoteAddr();
        }

        // 如果地址存在多个则获取第一个
        int separatorIndex = ip.indexOf(',');
        if (separatorIndex > -1) {
            ip = ip.substring(0, separatorIndex);
        }

        return HostUtils.DEFAULT_LOCAL_ADDRESS_IPV6.equals(ip) ? HostUtils.LOCAL_ADDRESS : ip;
    }

    /**
     * 获取Body请求参数
     *
     * @param request Http请求对象
     * @return 请求体
     * @throws IOException I/O异常
     */
    public static String getBody(@NonNull HttpServletRequest request) throws IOException {
        String line;
        StringBuilder buffer = new StringBuilder();
        try (BufferedReader reader = new BufferedReader(new InputStreamReader(request.getInputStream()))) {
            while ((line = reader.readLine()) != null) {
                buffer.append(line);
            }
        }
        return buffer.toString();
    }

    /**
     * 获取表单请求参数
     *
     * @param request Http请求对象
     * @return 参数键/值映射表
     */
    public static Map<String, String[]> getParameters(@NonNull HttpServletRequest request) {
        // 获取表单参数
        Map<String, String[]> parameters = Maps.newHashMap();
        Enumeration<String> enumeration = request.getParameterNames();
        while (enumeration.hasMoreElements()) {
            String name = enumeration.nextElement();
            if (StringUtils.nonEmpty(name)) {
                parameters.put(name, request.getParameterValues(name));
            }
        }
        return parameters;
    }

    /**
     * 响应文件
     *
     * @param response Http响应对象
     * @param file     文件对象
     * @throws IOException I/O异常
     */
    public static void write(@NonNull HttpServletResponse response, @NonNull File file) throws IOException {
        write(response, file, file.getName());
    }

    /**
     * 响应文件
     *
     * @param response Http响应对象
     * @param file     文件对象
     * @param filename 文件名称
     * @throws IOException I/O异常
     */
    public static void write(@NonNull HttpServletResponse response, @NonNull File file, @NonNull String filename)
            throws IOException {
        // 设置响应头
        response.reset();
        response.addHeader("Content-Length", String.valueOf(file.length()));
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(filename, "utf-8"));
        response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
        response.addHeader("Access-Control-Allow-Origin", "*");
        response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);

        // 将文件写入响应流
        int length;
        byte[] buffer = new byte[2048];
        try (InputStream input = new FileInputStream(file); OutputStream output = response.getOutputStream()) {
            while ((length = input.read(buffer)) > 0) {
                output.write(buffer, 0, length);
            }
            output.flush();
        }
    }

    /**
     * 对象URL编码
     *
     * @param original 原始信息
     * @return 对象编码结果
     */
    public static String encode(String original) {
        if (StringUtils.isEmpty(original)) {
            return StringUtils.EMPTY;
        }
        try {
            return URLEncoder.encode(original, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 对象URL解码
     *
     * @param encode 编码信息
     * @return 对象编码结果
     */
    public static String decode(String encode) {
        if (StringUtils.isEmpty(encode)) {
            return encode;
        }
        try {
            return URLDecoder.decode(encode, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * URL地址拼接
     *
     * @param domain 域名
     * @param uris   资源地址数组
     * @return URL地址
     */
    public static String url(@NonNull String domain, @NonNull String... uris) {
        if (uris == null || uris.length == 0) {
            return domain;
        }
        Matcher matcher = URI_SEPARATOR_PATTERN.matcher(StringUtils.join(uris, StringUtils.URI_SEPARATOR));
        String uri = matcher.replaceAll(StringUtils.URI_SEPARATOR);
        if (domain.charAt(domain.length() - 1) == StringUtils.URI_SEPARATOR.charAt(0)) {
            if (uri.charAt(0) == StringUtils.URI_SEPARATOR.charAt(0)) {
                return domain.concat(uri.substring(1));
            }
            return domain.concat(uri);
        } else if (uri.charAt(0) == StringUtils.URI_SEPARATOR.charAt(0)) {
            return domain.concat(uri);
        }
        return domain.concat(StringUtils.URI_SEPARATOR).concat(uri);
    }

    /**
     * Http请求参数序列化
     *
     * @param request Http请求对象
     * @return 请求参数
     * @throws IOException I/O异常
     */
    public static String serialize(@NonNull HttpServletRequest request) throws IOException {
        return serialize(request, false);
    }

    /**
     * Http请求参数序列化
     *
     * @param request  Http请求对象
     * @param ordering 参数是否排序
     * @return 请求参数
     * @throws IOException I/O异常
     */
    public static String serialize(@NonNull HttpServletRequest request, boolean ordering) throws IOException {
        // Body参数
        String type = request.getContentType();
        if (StringUtils.nonEmpty(type) && type.contains(MediaType.APPLICATION_JSON_VALUE)) {
            return getBody(request);
        }

        // 表单参数
        return StringUtils.join(getParameters(request), "&", (key, values) -> {
            if (values == null || values.length == 0) {
                return key + StringUtils.EQUAL;
            } else if (values.length == 1) {
                return key + StringUtils.EQUAL + StringUtils.ifEmpty(values[0], StringUtils.EMPTY);
            }
            StringBuilder buffer = new StringBuilder();
            for (String value : values) {
                if (StringUtils.nonEmpty(value)) {
                    if (buffer.length() > 0) {
                        buffer.append("&");
                    }
                    buffer.append(key).append(StringUtils.EQUAL).append(value);
                }
            }
            return buffer.length() == 0 ? (key + StringUtils.EQUAL) : buffer.toString();
        }, ordering);
    }
}
