package com.chelvc.framework.base.util;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Objects;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import lombok.NonNull;
import com.chelvc.framework.common.util.StringUtils;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.core.io.support.ResourcePatternResolver;
import org.springframework.core.type.classreading.CachingMetadataReaderFactory;
import org.springframework.core.type.classreading.MetadataReader;
import org.springframework.core.type.classreading.MetadataReaderFactory;
import org.springframework.util.AntPathMatcher;
import org.springframework.util.ClassUtils;

/**
 * Spring工具类
 *
 * @author Woody
 * @date 2023/1/1
 */
public final class SpringUtils {
    /**
     * 资源路径匹配器
     */
    private static final AntPathMatcher PATH_MATCHER = new AntPathMatcher(File.separator);

    private SpringUtils() {
    }

    /**
     * 判断资源路径是否符合指定匹配模式
     *
     * @param path     资源路径
     * @param patterns 匹配模式数组
     * @return true/false
     */
    public static boolean isPath(String path, String... patterns) {
        if (StringUtils.isEmpty(path) || patterns == null || patterns.length == 0) {
            return false;
        }
        for (String pattern : patterns) {
            if (StringUtils.nonEmpty(pattern) && PATH_MATCHER.match(pattern, path)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 查找工程内部的对象类型
     *
     * @param pack 包名
     * @return 类对象列表
     */
    public static List<Class<?>> lookupClasses(@NonNull String pack) {
        return lookupClasses(pack, clazz -> true);
    }

    /**
     * 查找工程内部的对象类型
     *
     * @param pack      包名
     * @param condition 类对象条件判断
     * @return 类对象列表
     */
    public static List<Class<?>> lookupClasses(@NonNull String pack, @NonNull Predicate<Class<?>> condition) {
        ResourcePatternResolver resourcePatternResolver = new PathMatchingResourcePatternResolver();
        String pattern = (pack = pack.trim()).isEmpty() ? "**/*.class" : (
                ResourcePatternResolver.CLASSPATH_ALL_URL_PREFIX
                        + ClassUtils.convertClassNameToResourcePath(pack)
                        + "/**/*.class"
        );

        // 获取对象资源文件
        Resource[] resources;
        try {
            resources = resourcePatternResolver.getResources(pattern);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        // 将资源文件转换为类对象
        Stream.Builder<Class<?>> classes = Stream.builder();
        MetadataReaderFactory readerFactory = new CachingMetadataReaderFactory(resourcePatternResolver);
        ClassLoader classLoader = Objects.requireNonNull(ClassUtils.getDefaultClassLoader(), "Class loader not found");
        for (Resource resource : resources) {
            try {
                // 加载类对象
                MetadataReader reader = readerFactory.getMetadataReader(resource);
                Class<?> clazz = classLoader.loadClass(reader.getClassMetadata().getClassName());
                if (clazz != null && condition.test(clazz)) {
                    classes.add(clazz);
                }
            } catch (IOException | ClassNotFoundException e) {
                throw new RuntimeException();
            }
        }
        return classes.build().distinct().collect(Collectors.toList());
    }
}
