package com.chelvc.framework.oauth.context;

import java.text.ParseException;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import javax.servlet.ServletRequestEvent;
import javax.servlet.ServletRequestListener;
import javax.servlet.http.HttpServletRequest;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.function.Adapter;
import com.chelvc.framework.common.model.Terminal;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.oauth.config.OauthProperties;
import com.google.common.collect.Sets;
import com.nimbusds.jwt.JWT;
import com.nimbusds.jwt.JWTClaimsSet;
import com.nimbusds.jwt.JWTParser;
import lombok.NonNull;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.common.OAuth2RefreshToken;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.security.oauth2.provider.ClientDetails;
import org.springframework.security.oauth2.server.resource.web.BearerTokenResolver;
import org.springframework.security.oauth2.server.resource.web.DefaultBearerTokenResolver;
import org.springframework.stereotype.Component;

/**
 * OAuth上下文工具类
 *
 * @author Woody
 * @date 2023/1/1
 */
@Component
public class OauthContextHolder implements ServletRequestListener {
    /**
     * Jwt主体身份标识
     */
    public static final String JWT_CLAIM_ID = "id";

    /**
     * Jwt身份标识
     */
    public static final String JWT_CLAIM_JTI = "jti";

    /**
     * Jwt应用范围
     */
    public static final String JWT_CLAIM_SCOPE = "scope";

    /**
     * Jwt手机号标识
     */
    public static final String JWT_CLAIM_MOBILE = "mobile";

    /**
     * Jwt用户账号标识
     */
    public static final String JWT_CLAIM_USER_NAME = "user_name";

    /**
     * Jwt用户权限标识
     */
    public static final String JWT_CLAIM_AUTHORITIES = "authorities";

    /**
     * 配置属性
     */
    private static OauthProperties PROPERTIES;

    /**
     * Jwt上下文
     */
    private static final ThreadLocal<Jwt> JWT_CONTEXT = new ThreadLocal<>();

    /**
     * 认证主体上下文
     */
    private static final ThreadLocal<Object> PRINCIPAL_CONTEXT = new ThreadLocal<>();

    /**
     * 客户端详情上下文
     */
    private static final ThreadLocal<ClientDetails> CLIENT_DETAILS_CONTEXT = new ThreadLocal<>();

    /**
     * 刷新令牌上下文
     */
    private static final ThreadLocal<OAuth2RefreshToken> REFRESH_TOKEN_CONTEXT = new ThreadLocal<>();

    /**
     * Bearer令牌解析器
     */
    private static final BearerTokenResolver BEARER_TOKEN_RESOLVER = new DefaultBearerTokenResolver();

    /**
     * 获取配置属性
     *
     * @return 配置属性
     */
    public static OauthProperties getProperties() {
        if (PROPERTIES == null) {
            synchronized (OauthProperties.class) {
                PROPERTIES = ApplicationContextHolder.getBean(OauthProperties.class);
            }
        }
        return PROPERTIES;
    }

    /**
     * 获取Jwt
     *
     * @return Jwt实例
     */
    public static Jwt getJwt() {
        return JWT_CONTEXT.get();
    }

    /**
     * 设置Jwt
     *
     * @param jwt Jwt实例
     */
    public static void setJwt(Jwt jwt) {
        JWT_CONTEXT.set(jwt);
    }

    /**
     * 获取认证主体
     *
     * @return 认证主体
     */
    public static Object getPrincipal() {
        return PRINCIPAL_CONTEXT.get();
    }

    /**
     * 设置认证主体
     *
     * @param principal 认证主体
     */
    public static void setPrincipal(Object principal) {
        PRINCIPAL_CONTEXT.set(principal);
    }

    /**
     * 获取客户端详情
     *
     * @return 客户端详情
     */
    public static ClientDetails getClientDetails() {
        return CLIENT_DETAILS_CONTEXT.get();
    }

    /**
     * 设置客户端详情
     *
     * @param details 客户端详情
     */
    public static void setClientDetails(ClientDetails details) {
        CLIENT_DETAILS_CONTEXT.set(details);
    }

    /**
     * 获取当前刷新令牌
     *
     * @return 刷新令牌实例
     */
    public static OAuth2RefreshToken getRefreshToken() {
        return REFRESH_TOKEN_CONTEXT.get();
    }

    /**
     * 设置当前刷新令牌
     *
     * @param refreshToken 刷新令牌实例
     */
    public static void setRefreshToken(OAuth2RefreshToken refreshToken) {
        REFRESH_TOKEN_CONTEXT.set(refreshToken);
    }

    /**
     * 令牌转换
     *
     * @param token 令牌信息
     * @return JWT对象实例
     */
    public static JWT parse(String token) {
        if (StringUtils.isEmpty(token)) {
            return null;
        }
        try {
            return JWTParser.parse(token);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 令牌转换
     *
     * @param request Http请求对象
     * @return JWT对象实例
     */
    public static JWT parse(HttpServletRequest request) {
        return StringUtils.ifEmpty(getToken(request), OauthContextHolder::parse);
    }

    /**
     * 获取Jwt属性集合
     *
     * @param jwt JWT对象实例
     * @return Jwt属性集合
     */
    public static JWTClaimsSet getJwtClaims(JWT jwt) {
        if (Objects.isNull(jwt)) {
            return null;
        }
        try {
            return jwt.getJWTClaimsSet();
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取Jwt属性值
     *
     * @param jwt      JWT对象实例
     * @param function 属性获取方法
     * @param <T>      属性值类型
     * @return 属性值
     */
    public static <T> T getJwtClaim(JWT jwt, @NonNull Adapter<JWTClaimsSet, T> function) {
        JWTClaimsSet claims = ObjectUtils.ifNull(jwt, OauthContextHolder::getJwtClaims);
        if (claims == null) {
            return null;
        }
        try {
            return function.apply(claims);
        } catch (Exception e) {
            if (e instanceof RuntimeException) {
                throw (RuntimeException) e;
            }
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取请求令牌
     *
     * @return 令牌信息
     */
    public static String getToken() {
        return getToken(SessionContextHolder.getRequest());
    }

    /**
     * 获取请求令牌
     *
     * @param request Http请求对象
     * @return 令牌信息
     */
    public static String getToken(HttpServletRequest request) {
        return StringUtils.ifEmpty(ObjectUtils.ifNull(request, BEARER_TOKEN_RESOLVER::resolve), (String) null);
    }

    /**
     * 获取主体身份
     *
     * @param jwt JWT对象
     * @return 主体身份标识
     */
    public static Long getId(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(JWT_CLAIM_USER_NAME), Long::parseLong);
    }

    /**
     * 获取主体身份
     *
     * @param jwt JWT对象
     * @return 主体身份标识
     */
    public static Long getId(JWT jwt) {
        return getJwtClaim(jwt, claims -> StringUtils.ifEmpty(
                claims.getStringClaim(JWT_CLAIM_USER_NAME), Long::parseLong
        ));
    }

    /**
     * 获取JWT身份标识
     *
     * @param jwt JWT对象
     * @return JWT身份标识
     */
    public static String getJti(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(JWT_CLAIM_JTI), (String) null);
    }

    /**
     * 获取JWT身份标识
     *
     * @param jwt JWT对象
     * @return JWT身份标识
     */
    public static String getJti(JWT jwt) {
        return StringUtils.ifEmpty(getJwtClaim(jwt, claims -> claims.getStringClaim(JWT_CLAIM_JTI)), (String) null);
    }

    /**
     * 获取应用范围
     *
     * @param jwt Jwt对象
     * @return 应用范围
     */
    public static String getScope(Jwt jwt) {
        List<String> scopes = jwt == null ? null : jwt.getClaimAsStringList(JWT_CLAIM_SCOPE);
        return StringUtils.ifEmpty(ObjectUtils.isEmpty(scopes) ? null : scopes.get(0), (String) null);
    }

    /**
     * 获取应用范围
     *
     * @param jwt JWT对象
     * @return 应用范围
     */
    public static String getScope(JWT jwt) {
        return StringUtils.ifEmpty(getJwtClaim(jwt, claims -> {
            String[] scopes = claims.getStringArrayClaim(JWT_CLAIM_SCOPE);
            return ObjectUtils.isEmpty(scopes) ? null : scopes[0];
        }), (String) null);
    }

    /**
     * 获取应用范围
     *
     * @param token 访问令牌
     * @return 应用范围
     */
    public static String getScope(OAuth2AccessToken token) {
        return getScope(parse(ObjectUtils.ifNull(token, OAuth2AccessToken::getValue)));
    }

    /**
     * 获取应用范围
     *
     * @param token 刷新令牌
     * @return 应用范围
     */
    public static String getScope(OAuth2RefreshToken token) {
        return getScope(parse(ObjectUtils.ifNull(token, OAuth2RefreshToken::getValue)));
    }

    /**
     * 获取手机号
     *
     * @param jwt JWT对象
     * @return 手机号
     */
    public static String getMobile(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(JWT_CLAIM_MOBILE), (String) null);
    }

    /**
     * 获取手机号
     *
     * @param jwt JWT对象
     * @return 手机号
     */
    public static String getMobile(JWT jwt) {
        return StringUtils.ifEmpty(getJwtClaim(jwt, claims -> claims.getStringClaim(JWT_CLAIM_MOBILE)), (String) null);
    }

    /**
     * 获取用户授权信息
     *
     * @param jwt JWT对象
     * @return 授权信息集合
     */
    public static Set<String> getAuthorities(Jwt jwt) {
        if (jwt == null) {
            return Collections.emptySet();
        }
        List<String> authorities = jwt.getClaimAsStringList(JWT_CLAIM_AUTHORITIES);
        return ObjectUtils.isEmpty(authorities) ? Collections.emptySet() : Sets.newHashSet(authorities);
    }

    /**
     * 获取用户授权信息
     *
     * @param jwt JWT对象
     * @return 授权信息集合
     */
    public static Set<String> getAuthorities(JWT jwt) {
        String[] authorities = getJwtClaim(jwt, claims -> claims.getStringArrayClaim(JWT_CLAIM_AUTHORITIES));
        return ObjectUtils.isEmpty(authorities) ? Collections.emptySet() : Sets.newHashSet(authorities);
    }

    /**
     * 构建访问令牌标识
     *
     * @param principal 主体标识
     * @param terminal  终端信息
     * @return 令牌标识
     */
    public static String buildAccessTokenKey(@NonNull Object principal, @NonNull Terminal terminal) {
        return String.format("token:%s:%s", terminal, principal);
    }

    /**
     * 清空OAuth上下文
     */
    public static void clearOauthContext() {
        JWT_CONTEXT.remove();
        PRINCIPAL_CONTEXT.remove();
        CLIENT_DETAILS_CONTEXT.remove();
        REFRESH_TOKEN_CONTEXT.remove();
    }

    @Override
    public void requestDestroyed(ServletRequestEvent event) {
        clearOauthContext();
    }
}
