package com.chelvc.framework.security.config;

import java.io.Serializable;
import java.util.Objects;
import java.util.stream.Stream;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Platform;
import com.chelvc.framework.common.model.Terminal;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import org.springframework.security.access.PermissionEvaluator;
import org.springframework.security.access.expression.SecurityExpressionRoot;
import org.springframework.security.access.expression.method.MethodSecurityExpressionOperations;
import org.springframework.security.access.hierarchicalroles.RoleHierarchy;
import org.springframework.security.authentication.AuthenticationTrustResolver;
import org.springframework.security.core.Authentication;

/**
 * 方法权限表达式处理包装类，自定义hasAuthority()、hasAnyAuthority()的处理逻辑
 *
 * @author Woody
 * @date 2023/1/1
 * @see SecurityConfigurer
 */
public class MethodSecurityExpression implements MethodSecurityExpressionOperations {
    private final SecurityExpressionRoot root;
    private Object filterObject;
    private Object returnObject;
    private Object target;

    public MethodSecurityExpression(Authentication authentication) {
        this.root = new SecurityExpressionRoot(authentication) {
        };
    }

    @Override
    public Object getFilterObject() {
        return this.filterObject;
    }

    @Override
    public void setFilterObject(Object filterObject) {
        this.filterObject = filterObject;
    }

    @Override
    public Object getReturnObject() {
        return this.returnObject;
    }

    @Override
    public void setReturnObject(Object returnObject) {
        this.returnObject = returnObject;
    }

    @Override
    public Object getThis() {
        return this.target;
    }

    public void setThis(Object target) {
        this.target = target;
    }

    @Override
    public Authentication getAuthentication() {
        return this.root.getAuthentication();
    }

    @Override
    public boolean hasAuthority(String authority) {
        return this.root.hasAuthority(authority);
    }

    @Override
    public boolean hasAnyAuthority(String... authorities) {
        return this.root.hasAnyAuthority(authorities);
    }

    @Override
    public boolean hasRole(String role) {
        return this.root.hasRole(role);
    }

    @Override
    public boolean hasAnyRole(String... roles) {
        return this.root.hasAnyRole(roles);
    }

    @Override
    public boolean permitAll() {
        return this.root.permitAll();
    }

    @Override
    public boolean denyAll() {
        return this.root.denyAll();
    }

    @Override
    public boolean isAnonymous() {
        return this.root.isAnonymous();
    }

    @Override
    public boolean isAuthenticated() {
        return this.root.isAuthenticated();
    }

    @Override
    public boolean isRememberMe() {
        return this.root.isRememberMe();
    }

    @Override
    public boolean isFullyAuthenticated() {
        return this.root.isFullyAuthenticated();
    }

    @Override
    public boolean hasPermission(Object target, Object permission) {
        return this.root.hasPermission(target, permission);
    }

    @Override
    public boolean hasPermission(Object targetId, String targetType, Object permission) {
        return this.root.hasPermission(targetId, targetType, permission);
    }

    public void setPermissionEvaluator(PermissionEvaluator permissionEvaluator) {
        this.root.setPermissionEvaluator(permissionEvaluator);
    }

    public void setTrustResolver(AuthenticationTrustResolver trustResolver) {
        this.root.setTrustResolver(trustResolver);
    }

    public void setRoleHierarchy(RoleHierarchy roleHierarchy) {
        this.root.setRoleHierarchy(roleHierarchy);
    }

    public void setDefaultRolePrefix(String defaultRolePrefix) {
        this.root.setDefaultRolePrefix(defaultRolePrefix);
    }

    /**
     * 判断是否是指定应用范围
     *
     * @param scopes 应用范围数组
     * @return true/false
     */
    public boolean isScope(String... scopes) {
        if (ObjectUtils.isEmpty(scopes)) {
            return false;
        }
        String current = SessionContextHolder.getScope();
        return StringUtils.nonEmpty(current) && Stream.of(scopes).filter(StringUtils::nonEmpty)
                .anyMatch(type -> Objects.equals(type, current));
    }

    /**
     * 判断是否是指定手机号
     *
     * @param mobiles 手机号数组
     * @return true/false
     */
    public boolean isMobile(String... mobiles) {
        if (ObjectUtils.isEmpty(mobiles)) {
            return false;
        }
        String current = SessionContextHolder.getMobile();
        return StringUtils.nonEmpty(current) && Stream.of(mobiles).filter(StringUtils::nonEmpty)
                .anyMatch(mobile -> Objects.equals(mobile, current));
    }

    /**
     * 判断是否是指定ID
     *
     * @param identities ID数组
     * @return true/false
     */
    public boolean isIdentity(Serializable... identities) {
        if (ObjectUtils.isEmpty(identities)) {
            return false;
        }
        Serializable current = SessionContextHolder.getId();
        return StringUtils.nonEmpty(current) && Stream.of(identities).filter(StringUtils::nonEmpty)
                .anyMatch(identity -> Objects.equals(identity, current));
    }

    /**
     * 判断是否是指定平台
     *
     * @param platforms 平台数组
     * @return true/false
     */
    public boolean isPlatform(Platform... platforms) {
        if (ObjectUtils.isEmpty(platforms)) {
            return false;
        }
        Platform current = SessionContextHolder.getPlatform();
        return current != null && Stream.of(platforms).filter(Objects::nonNull)
                .anyMatch(platform -> Objects.equals(platform, current));
    }

    /**
     * 判断是否是指定终端
     *
     * @param terminals 终端数组
     * @return true/false
     */
    public boolean isTerminal(Terminal... terminals) {
        if (ObjectUtils.isEmpty(terminals)) {
            return false;
        }
        Terminal current = SessionContextHolder.getTerminal();
        return current != null && Stream.of(terminals).filter(StringUtils::nonEmpty)
                .anyMatch(terminal -> Objects.equals(terminal, current));
    }

    /**
     * 判断是否是指定版本
     *
     * @param versions 版本号数组
     * @return true/false
     */
    public boolean isVersion(String... versions) {
        if (ObjectUtils.isEmpty(versions)) {
            return false;
        }
        String current = SessionContextHolder.getVersion();
        return StringUtils.nonEmpty(current) && Stream.of(versions).filter(Objects::nonNull)
                .anyMatch(version -> Objects.equals(version, current));
    }
}
