package com.chelvc.framework.security.interceptor;

import java.io.IOException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.ContextualSerializer;
import com.fasterxml.jackson.databind.ser.std.StdSerializer;
import lombok.extern.slf4j.Slf4j;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.DesensitizeUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.security.annotation.Desensitize;

/**
 * 敏感数据脱敏序列化处理器
 *
 * @author Woody
 * @date 2023/1/1
 */
@Slf4j
public class JacksonDesensitizeSerializer extends StdSerializer<String> implements ContextualSerializer {
    /**
     * 敏感数据脱敏表达式/脱敏模型映射表
     */
    private static final Map<String, DesensitizeUtils.Mode> DESENSITIZE_MODE_MAPPING = new ConcurrentHashMap<>();

    private boolean enabled;
    private Desensitize annotation;

    public JacksonDesensitizeSerializer() {
        super(String.class);
    }

    /**
     * 判断是否受版本控制
     *
     * @param annotation 脱敏注解实例
     * @return true/false
     */
    private boolean isVersioning(Desensitize annotation) {
        return ObjectUtils.isEmpty(annotation.versioning()) || SessionContextHolder.isVersion(annotation.versioning());
    }

    /**
     * 判断是否是可脱敏属性
     *
     * @param property 属性对象
     * @return true/false
     */
    private boolean isDesensitizeProperty(BeanProperty property) {
        return property.getType().getRawClass() == String.class;
    }

    /**
     * 获取属性脱敏注解实例
     *
     * @param property 属性对象
     * @return 脱敏注解实例
     */
    private Desensitize getDesensitizeAnnotation(BeanProperty property) {
        return property.getAnnotation(Desensitize.class);
    }

    /**
     * 获取数据脱敏模式
     *
     * @param annotation 脱敏注解实例
     * @return 数据脱敏模式
     */
    private DesensitizeUtils.Mode getDesensitizeMode(Desensitize annotation) {
        String expression = ObjectUtils.ifNull(annotation, Desensitize::expression);
        if (StringUtils.isEmpty(expression)) {
            return null;
        }
        return DESENSITIZE_MODE_MAPPING.computeIfAbsent(expression, DesensitizeUtils::analyse);
    }

    @Override
    public void serialize(String plaintext, JsonGenerator generator, SerializerProvider provider)
            throws IOException {
        DesensitizeUtils.Mode mode;
        if (!this.enabled || this.annotation == null || StringUtils.isEmpty(plaintext)
                || (mode = this.getDesensitizeMode(this.annotation)) == null) {
            generator.writeString(plaintext);
        } else {
            String ciphertext = mode.desensitize(plaintext);
            if (log.isDebugEnabled()) {
                log.debug("Data desensitize: {} -> {}", plaintext, ciphertext);
            }
            generator.writeString(ciphertext);
        }
    }

    @Override
    public JsonSerializer<?> createContextual(SerializerProvider provider, BeanProperty property) {
        if (property == null) {
            return provider.getDefaultNullValueSerializer();
        }
        if (this.isDesensitizeProperty(property)) {
            this.annotation = this.getDesensitizeAnnotation(property);
            this.enabled = this.annotation != null && this.isVersioning(this.annotation);
        }
        return this;
    }
}
